#ifndef __COLLISIONOBJECT_H__
#define __COLLISIONOBJECT_H__

#ifdef __APPLE__
	#include <GraphicsLib/Point3.h>
    #include <GraphicsLib/types.h>
#else
	#include "Point3.h"
    #include "types.h"
#endif

/**
 * Name: CollisionObject.h
 *
 * Copyright (c) 2005 University of Bournemouth
 */
#include "Particle.h"

namespace ParticleSystems {

//! The CollisionObject Class
/**
* Super Class to handle Collisions. This class defines
* the abstract methods that need to be implemented by any 
* sub Class. 
* @author Hannes Ricklefs
*/
class CollisionObject {
	public:
		//! The Constructor
		/**
		* Constructs a new CollisionObject
		*/
		CollisionObject();
		
		//! The Destructor
		/**
		* Destructs a CollisionObject
		*/
		virtual ~CollisionObject();
		
		//! A public virtual function
		/**
		* Checks if a collision is occured between
		* this CollisionObject and the passed Particle
		* \param p A pointer to a particle
		*/
		virtual void collide(Particle *p) = 0;
		
		//! A public virtual function
		/**
		* Draws this collision object.
		*/
        virtual void draw() = 0;
		
		//! A public virtual function
		/**
		* Translates this collision object.
		*/
        virtual void translate() = 0;
		
		//! A public virtual function
		/**
		* Rotates this collision object.
		*/
        virtual void rotate() = 0;
		
		//! A public virtual function
		/**
		* scales this collision object.
		*/
        virtual void scale() = 0;
		
		//! A public variable
		/**
		* The distance between this Collision object and the object to be
		* checked against.
		*/
        GraphicsLib::Real distance;
	};
}
#endif

