/**
 * Name: Force.h
 *
 * Copyright (c) 2005 University of Bournemouth
 */
#ifndef __FORCE_H__
#define __FORCE_H__

#ifdef __APPLE__
#include <GraphicsLib/Vector.h>
#else
#include "Vector.h"
#endif

namespace ParticleSystems {
	//! The Force Class
	/**
	* The Force Class is the Superclass for all Forces. Forces influence the 
	* movement of the Particles in a per Force specific way.
	* @author Hannes Ricklefs
	*/
	class Force {
	public:
		//! The Constructor
		/**
		* The Force Constructor takes a Vector that defines the direction
		* on how to influence the Particle accoriding to a given attenuation.
		* If the attenuation is set to 1 the Force will be fully applied
		* \param f a pointer to a direction Vector
		* \param a the attenuation
		*/
		Force(GraphicsLib::Vector *f, float a);
			
		//! The Destructor
		/**
		* Destructs the Force
		*/
		virtual ~Force();
			
		//! An abstract Function
		/**
		* Applies this Force to the given direction Vector.
		* \param direction The direction Vector to be affected by this Force
		*/
		virtual void applyForce(GraphicsLib::Vector *direction) = 0;
			
		//! An abstract Function
		/**
		* Applies this Force to the given direction Vector taking into consideration
		* a given Mass.
		* \param direction The direction Vector to be affected by this Force
		* \param mass a pointer to the mass
		*/
		virtual void applyForce(GraphicsLib::Vector *direction, float *mass) = 0;
			
		//! A public Variable
		/**
		* The direction of this Force
		*/
		GraphicsLib::Vector *force;
			
		//! A public Variable
		/**
		* The attenuation of the Force
		*/
		float attenuation;
	};
}
#endif
