/**
 * Name: ImgParticle.h
 *
 * Copyright (c) 2005 University of Bournemouth
 */
#ifndef __IMGPARTICLE_H__
#define __IMGPARTICLE_H__

#ifdef __APPLE__
	#include <GLUT/glut.h>
#else
	#include <GL/glut.h>
#endif

#include "Particle.h"
#include "Texture.h"
#include "TGALoader.h"
#include "Debugger.h"

namespace ParticleSystems {
	//! The ImgParticle Class
	/**
	* The ImgParticle Class is a Subclass of the Particle Class.
	* It extends the Particle Class in order to be able to texture
	* a Particle with a Texture.
	* @see Particle
	* @author Hannes Ricklefs
	*/
	class ImgParticle : public Particle {
	public:
		
		//! The minimal Constructor
		/**
		* This Constructor only takes the ParticleSystem ID, the 
		* Solver and the Textureof a Particle.
		* \param sysId the ParticleSystem ID this ImgParticle belongs to
		* \param sol a pointer to the Solver
		* \param text a pointer to a Texture
		*/
		ImgParticle(int sysId, Solver *sol,Texture *text);
		
		//! The main Constructor
		/**
		* This Constructor is the main Constructor for a ImgParticle. It sets the 
		* most important attributes of a ImgParticle. 
		* \param sysId the ParticleSystem ID this Particle belongs to
		* \param sol a pointer to the Solver
		* \param pos the initial Position
		* \param vel the inital Velocity
		* \param c the initial Colour
		* \param fc the fade Colour
		* \param size the inital Size
		* \param fsize the fade Size
		* \param maxLife the maximum Lifespan of the Particle
		* \param text a pointer to a Texture
		*/
		ImgParticle(int sysId, 
					Solver *sol, 
					GraphicsLib::Point3 pos, 
					GraphicsLib::Vector vel, 
					GraphicsLib::Colour c,
					GraphicsLib::Colour fc,
					GLfloat size,
					GLfloat fs,
					GLfloat maxLife,
					Texture *text);
					
		//! The Destructor
		/**
		* Destructs the ImgParticle
		*/
		virtual ~ImgParticle();
		
		//! A virtual Function
		/**
		* Draws the ImgParticle
		*/
		virtual void draw(void);
		
		//! A virtual Function
		/**
		* Updates the ImgParticles
		* attributes 
		*/
		virtual void update(void);
		
		//! A virtual Function
		/**
		* Returns true if the ImgParticle is dead
		* \return the value if the ImgParticle is dead
		*/
		virtual bool isDead(void);	
		
		//! A public Variable
		/**
		* The Pointer to the Texture for the ImgParticle
		*/
		Texture *texture;		
	private :
		//! A private Variable
		/**
		* The TextureID needed for binding the Texture
		*/
		unsigned int TextureID;
	};
}
#endif
