#ifndef __PLANAREMITTER_H__
#define __PLANAREMITTER_H__

/**
 * Name: PlanarEmitter.h
 *
 * Copyright (c) 2005 University of Bournemouth
 */
#ifdef __APPLE__
#include <GraphicsLib/Point3.h>
#include <GraphicsLib/Vector.h>
#else
#include "Vector.h"
#include "Point3.h"
#endif

#include "Emitter.h"
#include "PointParticle.h"
#include "ImgParticle.h"
#include "StrikeParticle.h"

namespace ParticleSystems {
	//! The Planar Class
	/**
	* The PlanarEmitter is a Subclass of the Emitter Class. It emits Particles 
	* randomly from anywhere on the specified Plane.
	* @author Hannes Ricklefs
	*/
	class PlanarEmitter : public Emitter {
	public:
		//! The Constructor
		/**
		* Constructs a new PlanarEmitter
		* \param SYSID the ParticleSystemID
		* \param type the ParticleType
		* \param initVel the initial velocity for a particle
		* \param initalColor the inital Colour of a particle
		* \param pSize the inital Size
		* \param dim1 the first Vector to define the plane
		* \param dim2 the second Vector to define the plane
		*/
		PlanarEmitter(	int SYSID, Solver *s, 
						PARTICLETYPE type, 
						GraphicsLib::Vector initVel, 
						GraphicsLib::Colour initialColor, 
						GLfloat pSize,
						GraphicsLib::Vector dim1, 
						GraphicsLib::Vector dim2);
		
		//! The Constructor
		/**
		* Constructs a new PlanarEmitter
		* \param SYSID the ParticleSystemID
		* \param type the ParticleType
		* \param initVel the initial velocity for a particle
		* \param initalColor the inital Colour of a particle
		* \param pSize the inital Size
		* \param TexName the texture
		* \param dim1 the first Vector to define the plane
		* \param dim2 the second Vector to define the plane
		*/
		PlanarEmitter(	int SYSID, 
						Solver *s, 
						PARTICLETYPE type, 
						GraphicsLib::Vector initVel, 
						GraphicsLib::Colour initialColor, 
						GLfloat pSize,
						char *TexName,  
						GraphicsLib::Vector dim1,  
						GraphicsLib::Vector dim2);
		
		//! The Destructor
		/**
		* Destructs this PlanarEmitter
		*/
		~PlanarEmitter();
		
		//! An virtual Function
		/**
		* Emitts a new Particle according to the values
		* \return A pointer to the new Particle
		*/
		virtual Particle* emittParticles(void);
		
		//! An virtual Function
		/**
		* Reinitialises the given Particle as it would be newly created
		* \param P a pointer to the Particle to be reborne
		*/
		virtual void reBirth(Particle *p);
		
		//! A member Function
		/**
		* Returns a random Point3 on the Plane
		* \return Point3 the random Point on the Plane
		*/
		GraphicsLib::Point3 pointOnPlane();
		
		//! A public Variable
		/**
		* The first dimension
		*/
		GraphicsLib::Vector dimension1;
		
		//! A public Variable
		/**
		* The second dimension
		*/
		GraphicsLib::Vector dimension2;
	};
}
#endif