/**
 * Name: RandomForce.h
 *
 * Copyright (c) 2005 University of Bournemouth
 */
#ifndef __RANDOMFORCE_H__
#define __RANDOMFORCE_H__

#ifdef __APPLE__
#include <GraphicsLib/Vector.h>
#include <GraphicsLib/Util.h>
#else
#include "Vector.h"
#include "Util.h"
#endif

#include "Force.h"

namespace ParticleSystems {
	//! The RandomForce Class
	/**
	* The RandomForce Class extends the Force Class. It affects the 
	* direction of a Particle randomly
	* @author Hannes Ricklefs
	*/
	class RandomForce : public Force {
	public:
		//! The Constructor
		/**
		* The RandomForce Constructor takes a Vector that defines the direction
		* on how to influence the Particle accoriding to a given attenuation.
		* If the attenuation is set to 1 the Force will be fully applied
		* \param f a pointer to a direction Vector
		* \param a the attenuation
		*/
		RandomForce(GraphicsLib::Vector *f, float a);
		
		//! The Destructor
		/**
		* Destructs the Force
		*/
		~RandomForce();
		
		//! A virtual Function
		/**
		* Applies this RandomForce to the given direction Vector.
		* \param direction The direction Vector to be affected by this RandomForce
		*/
		virtual void applyForce(GraphicsLib::Vector *direction);
		
		//! A virtual Function
		/**
		* Applies this RandomForce to the given direction Vector taking into consideration
		* a given Mass.
		* \param direction The direction Vector to be affected by this RandomForce
		* \param mass a pointer to the mass
		*/
		virtual void applyForce(GraphicsLib::Vector *direction, float *mass);
	};
}
#endif