/**
 * Name: StraightSolver.h
 *
 * Copyright (c) 2005 University of Bournemouth
 */
#ifndef __STRAIGHTSOLVER_H__
#define __STRAIGHTSOLVER_H__

#ifdef __APPLE__
#include <GraphicsLib/Vector.h>
#include <GraphicsLib/Point3.h>
#else
#include "Point3.h"
#include "Vector.h"
#endif

#include "Solver.h"
#include "Force.h"
#include "ParticleSystem.h"

namespace ParticleSystems {
	//! The StraightSolver Class
	/** 
	* The StraightSolver Class is a Subclass of the Solver Superclass. If no
	* Force is added to the StraightSolver it evolves the direction passed in
	* a straight line.
	* @author Hannes Ricklefs
	*/
	class StraightSolver : public Solver {
		public:
		
			//! The Constructor
			/**
			* The Constructor which adds the given Force to the
			* forces vector.
			* \param f a pointer to the Force
			*/
			StraightSolver(Force *f);
			
			//! The default Constructor
			/**
			* The default Constructor
			*/
			StraightSolver();
			
			//! The Destructor
			/**
			* The Destructor
			*/
			virtual ~StraightSolver();
			
			//! An abstract Function
			/**
			* Evolves the given direction Vector by applying the
			* Forces assigned to this Solver. 
			* \param direction a pointer to the direction Vector to change
			*/
			virtual void nextStep(GraphicsLib::Vector *direction);
			
			//! An abstract Function
			/**
			* Evolves the given direction Vector by applying the
			* Forces assigned to this Solver. This function also takes
			* into consideration the mass passed.
			* \param direction a pointer to the direction Vector to change
			* \param mass the mass of the Particle
			*/
			virtual void nextStep(GraphicsLib::Vector *direction, float *mass);
	};
}
#endif
