/**
 * Name: UniformForce.h
 *
 * Copyright (c) 2005 University of Bournemouth
 */
#ifndef __UNIFORMFORCE_H__
#define __UNIFORMFORCE_H__

#ifdef __APPLE__
#include <GraphicsLib/Vector.h>
#include <GraphicsLib/Util.h>
#else
#include "Util.h"
#include "Vector.h"
#endif

#include "Force.h"

namespace ParticleSystems {
	//! The UniformForce Class
	/**
	* The UniformForce Class extends the Force Class. It affects the 
	* direction of a Particle directly.
	* @author Hannes Ricklefs
	*/

	class UniformForce : public Force {
	public:
		//! The Constructor
		/**
		* The UniformForce Constructor takes a Vector that defines the direction
		* on how to influence the Particle accoriding to a given attenuation.
		* If the attenuation is set to 1 the Force will be fully applied
		* \param f a pointer to a direction Vector
		* \param a the attenuation
		*/
		UniformForce(GraphicsLib::Vector *f, float a);
		
		//! The Destructor
		/**
		* Destructs the Force
		*/
		~UniformForce();
		
		//! A virtual Function
		/**
		* Applies this UniformForce to the given direction Vector.
		* \param direction The direction Vector to be affected by this UniformForce
		*/
		virtual void applyForce(GraphicsLib::Vector *direction);
		
		//! A virtual Function
		/**
		* Applies this UniformForce to the given direction Vector taking into consideration
		* a given Mass.
		* \param direction The direction Vector to be affected by this UniformForce
		* \param mass a pointer to the mass
		*/
		virtual void applyForce(GraphicsLib::Vector *direction, float *mass);
	};
}
#endif